<?php

namespace Drupal\wcm_simplesamlphp_auth;

use Drupal\Core\Url;

/**
 * Functions for verifying and refreshing simpleSAMLPHP metadata.
 *
 *  $metadataFile - name of the metadata file.
 *  $paramFilePath - relative path to the parameters.php file.
 *  $maxMetadataAge - maximum age in seconds of the metadata file.
 */
class Metarefresh {

  private $metadataFile = '/files/private/simplesamlphp/metadata/saml20-idp-remote.php';
  private $parametersFile = '/files/private/simplesamlphp/config/parameters.php';
  private $cronModulePath = 'module.php/cron/run/hourly/';
  private $minTimeToExpire = 43200;

  /**
   * Check metadata expiration and refresh if needed.
   */
  public function refreshMetadata() {
    // Refresh flag.
    $refresh = FALSE;
    // Log time to expire.
    $timeToExpire = $this->getTimeToMetadataExpiration();
    $msg = sprintf(
      'SimpleSAMLphp metadata expires in %s seconds.',
      $timeToExpire
    );
    \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
    // If time to expire is less than _minTimeToExpire, refresh.
    if ($timeToExpire < $this->minTimeToExpire) {
      $this->requestMetadataRefresh();
    }
  }

  /**
   * Hit the metadata refresh URL.
   */
  private function requestMetadataRefresh() {
    // Default to HTTPS
    $httpSecure = TRUE;
    $httpPrefix = 'https';

    // Check if we're allowing http 
    $allowHttpRefresh = $this->getSimplesamlphpParameter('allowHttpRefresh');
    if ($allowHttpRefresh) {
      // Log the override setting.
      $msg = sprintf(
        'Simplesaml metadata refresh setting allowHttpRefresh is enabled.'
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
      // Determine HTTPS from server vars
      if (isset($_SERVER['HTTPS']) && !$_SERVER['HTTPS']) {
        $httpSecure = FALSE;
        $httpPrefix = 'http';
      }
    }

    // Set the cron module URL
    $baseURLpath = $this->getSimplesamlphpParameter('baseURLPath');
    $cronModuleUrl = trim($baseURLpath . $this->cronModulePath);

    // Make sure a valid refresh URL was found.
    if (!strlen(trim($cronModuleUrl)) || $cronModuleUrl == $this->cronModulePath) {
      // Log error and exit.
      $msg = "SimplesamlPHP metadata refresh failed. Could not determine a metadata refresh URL.";
      \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
      exit;
    }

    // Rewrite parameter if it was given an http URL and http is not allowed.
    if (!$allowHttpRefresh && substr($cronModuleUrl, 0, 5) != 'https') {
      $oldCronModuleUrl = $cronModuleUrl;
      $cronModuleUrl = str_replace('http', 'https', $cronModuleUrl);
      // Log the corrected url.
      $msg = sprintf(
        'Simplesaml metadata refresh requires https. Rewriting cronModuleUrl parameter from %s to %s',
        $oldCronModuleUrl,
        $cronModuleUrl,
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
    }

    // Get cronkey parameter.
    $cronKey = $this->getSimplesamlphpParameter('cronKey');
    // Append cronkey to URL
    $cronModuleUrl = $cronModuleUrl . $cronKey;
    // Create URL options array.
    $urlOptions = [
      'absolute' => TRUE,
      'https' => $httpSecure,
    ];
    
    // Construct refresh URL.
    $metaRefreshUrl = Url::fromUri($cronModuleUrl, $urlOptions)->toString();
    // Log the source and cron url.
    $msg = sprintf(
      'Using %s value for metaRefreshUrl: %s',
      $cronModuleUrlSource,
      $metaRefreshUrl
    );
    \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
    
    // Use curl to hit the cron endpoint and kick metarefresh.
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $metaRefreshUrl);
    $success = curl_exec($ch);
    // Get the return code.
    $curlReturnCode =  curl_getinfo($ch, CURLINFO_HTTP_CODE);
    // Check if curl was successful. 
    if ($success === FALSE) {
      $msg = sprintf(
        'Simplesamlphp metarefresh failed with http code %s',
        $curlReturnCode
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
    }
    // Report success when we get 200.
    elseif ($curlReturnCode == 200) {
      $msg = sprintf(
        'Simplesamlphp metarefresh succeeded. Returned http %s',
        $curlReturnCode
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
    }
    // Report anything other than 200 as an error.
    else {
      $msg = sprintf(
        'Simplesamlphp metarefresh result unclear. Returned http %s',
        $curlReturnCode
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
    }
    curl_close($ch);
  }

  /**
   * Returns the current metadata expiration.
   *
   * @return int
   *   Epoch time expiration value.
   */
  private function getTimeToMetadataExpiration() {
    // Default to expired.
    $timeToExpire = 0;
    // File exists. Get expire value.
    if (file_exists($this->metadataFile)) {
      include_once $this->metadataFile;
      $expires = $metadata['urn:mace:incommon:osu.edu']['expire'];
      $timeToExpire = $expires - time();
    }
    // Return time to expire.
    return $timeToExpire;
  }

  /**
   * Returns the current value for the passed parameter. Logs an error when param key or file not found.
   *
   * @param string $param
   *   String name of the parameter to fetch.
   *
   * @return string
   *   Value of the parameter.
   */
  private function getSimplesamlphpParameter($param) {
    $paramValue = "";
    $saml_parameters = [];
    if (file_exists($this->parametersFile)) {
      require $this->parametersFile;
      // Return requested value from parameter array.
      if (array_key_exists($param, $saml_parameters)) {
        $paramValue = $saml_parameters[$param];
      // Requested param doesn't exist.
      } else {
        $msg = sprintf(
          'SimpleSAMLphp parameter %s is not defined in %s',
          $param,
          $this->parametersFile
        );
        \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
      }
    // Parameters file doesn't exist
    } else {
      $msg = sprintf(
        'SimpleSAMLphp parameters files can not be found: %s',
        $this->parametersFile
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
    }
    return $paramValue;
  }

}
